


class_id = 1
class_names = ['person', 'bicycle']


for i in range(10):
    # check if class id is valid
    if (class_names != None) and (class_id >= len(class_names)):
        continue
    print("gggg")


#%%
from oneclickai.YOLO import stream, load_model

# example usage
model = load_model("YOLO_coco")

# coco dataset cls names
coco_cls_names = ['person', 'bicycle', 'car', 'motorcycle', 'airplane', 'bus', 'train', 'truck', 'boat',
                'traffic light', 'fire hydrant', 'stop sign', 'parking meter', 'bench', 'bird', 'cat', 'dog',
                'horse', 'sheep', 'cow', 'elephant', 'bear', 'zebra', 'giraffe', 'backpack', 'umbrella',
                'handbag', 'tie', 'suitcase', 'frisbee', 'skis', 'snowboard', 'sports ball', 'kite', 'baseball bat',
                'baseball glove', 'skateboard', 'surfboard', 'tennis racket', 'bottle', 'wine glass', 'cup', 'fork',
                'knife', 'spoon', 'bowl', 'banana', 'apple', 'sandwich', 'orange', 'broccoli', 'carrot', 'hot dog',
                'pizza', 'donut', 'cake', 'chair', 'couch', 'potted plant', 'bed', 'dining table', 'toilet', 'tv',
                'laptop', 'mouse', 'remote', 'keyboard', 'cell phone', 'microwave', 'oven', 'toaster', 'sink',
                'refrigerator', 'book', 'clock', 'vase', 'scissors', 'teddy bear', 'hair drier', 'toothbrush']

stream(model, conf=0.5, class_names=coco_cls_names, video_source=0)


from oneclickai.YOLO import load_model, predict, draw_result
import cv2
import numpy as np

model = load_model("YOLO_coco")

# image path
image = cv2.imread('./test/test5.jpg')/255.0

# coco dataset cls names
coco_cls_names = ['person', 'bicycle', 'car', 'motorcycle', 'airplane', 'bus', 'train', 'truck', 'boat',
                'traffic light', 'fire hydrant', 'stop sign', 'parking meter', 'bench', 'bird', 'cat', 'dog',
                'horse', 'sheep', 'cow', 'elephant', 'bear', 'zebra', 'giraffe', 'backpack', 'umbrella',
                'handbag', 'tie', 'suitcase', 'frisbee', 'skis', 'snowboard', 'sports ball', 'kite', 'baseball bat',
                'baseball glove', 'skateboard', 'surfboard', 'tennis racket', 'bottle', 'wine glass', 'cup', 'fork',
                'knife', 'spoon', 'bowl', 'banana', 'apple', 'sandwich', 'orange', 'broccoli', 'carrot', 'hot dog',
                'pizza', 'donut', 'cake', 'chair', 'couch', 'potted plant', 'bed', 'dining table', 'toilet', 'tv',
                'laptop', 'mouse', 'remote', 'keyboard', 'cell phone', 'microwave', 'oven', 'toaster', 'sink',
                'refrigerator', 'book', 'clock', 'vase', 'scissors', 'teddy bear', 'hair drier', 'toothbrush']



result_annotation = predict(model, image, conf=0.4)
result_image = draw_result(np.array(image), result_annotation, class_names = coco_cls_names)
cv2.imshow('image', result_image)


# Close the window when 'Esc' is pressed
if cv2.waitKey(0) & 0xFF == 27:
    cv2.destroyAllWindows()




from oneclickai.YOLO import fit_yolo_model

# training data path
train_data_path = './yolo_dataset'
train_label_path = './yolo_dataset'

# validation data path
val_data_path = './yolo_dataset'
val_label_path = './yolo_dataset'

# fit model
fit_yolo_model(train_data_path, train_label_path, val_data_path, val_label_path, epochs=30)


#%%
import numpy as np
import os
import cv2
import tensorflow as tf

from .readTxt import load_data_from_txt
from .labelEncode import encode
from .createDataAugumentation import random_horizontal_flip, random_color_jitter, random_crop, letterbox_and_pad


def data_generator(num_classes, high_stride, low_stride, img_size,
                   data_path, label_path, batch_size=32, shuffle=True, prob=0.5):

    # List image files (without extension)
    image_list = [f for f in os.listdir(data_path) if f.lower().endswith(('.jpg', '.jpeg', '.png'))]
    image_files = [os.path.splitext(f)[0] for f in image_list]
    
    num_images = len(image_files)
    indices = np.arange(num_images)

    # Preallocate label shapes
    high_label_shape = (high_stride, high_stride, num_classes + 5)
    low_label_shape = (low_stride, low_stride, num_classes + 5)

    # shuffle
    if shuffle:
        np.random.shuffle(indices)

    for start in range(0, num_images, batch_size):
        end = min(start + batch_size, num_images)
        batch_indices = indices[start:end]
        current_batch_size = len(batch_indices)
        
        # Preallocate arrays for the batch
        x_batch = np.empty((current_batch_size, img_size, img_size, 3), dtype=np.float32)
        y_batch_high = np.empty((current_batch_size, *high_label_shape), dtype=np.float32)
        y_batch_low = np.empty((current_batch_size, *low_label_shape), dtype=np.float32)
        
        for i, idx in enumerate(batch_indices):
            image_id = image_files[idx]

            # Load and process the image
            image_path = os.path.normpath(os.path.join(data_path, image_list[idx]))
            image = cv2.imread(image_path)

            # Load annotation
            label_file = os.path.join(label_path, image_id + '.txt')
            annotation = load_data_from_txt(label_file)

            # image resize
            # image = cv2.resize(image, (img_size, img_size))

            # pad if necessary with a probabiliy
            image, annotation = letterbox_and_pad(image, annotation, final_size=img_size, prob=prob)

            # Data Augmentation
            image, annotation = random_horizontal_flip(image, annotation, prob=prob)
            image = random_color_jitter(image, brightness_range=0.2, contrast_range=0.4, prob=prob)
            image, annotation = random_crop(image, annotation, scale_range=(0.5, 1.0), prob=prob)

            # Encode labels
            high_label, low_label = encode(
                annotation, num_classes, high_stride, low_stride, area_threshold=0.08
            )
            
            image = image / 255.0  # Normalize
            x_batch[i] = image
            y_batch_high[i] = high_label
            y_batch_low[i] = low_label
        

        yield x_batch, (y_batch_high, y_batch_low)


# Wrap the generator in a tf.data.Dataset:
def create_tf_dataset(num_classes, high_stride, low_stride, img_size,
                      data_path, label_path, batch_size=32, shuffle=True, prob=0.5):
    output_signature = (
        tf.TensorSpec(shape=(None, img_size, img_size, 3), dtype=tf.float32),
        (
            tf.TensorSpec(shape=(None, high_stride, high_stride, num_classes+5), dtype=tf.float32),
            tf.TensorSpec(shape=(None, low_stride, low_stride, num_classes+5), dtype=tf.float32)
        )
    )
    
    dataset = tf.data.Dataset.from_generator(
        lambda: data_generator(num_classes, high_stride, low_stride, img_size,
                                 data_path, label_path, batch_size, shuffle, prob),
        output_signature=output_signature
    )
    # Prefetch allows the next batch to be prepared while the current one is being processed.
    dataset = dataset.prefetch(buffer_size=tf.data.AUTOTUNE)
    return dataset


# #%%
# # Example usage:
# if __name__ == '__main__':
#     from drawImage import draw_result_imshow
#     import labelDecode
#     data_path = 'C:/Users/osy04/Desktop/wok_me/project/yolo/images/val2017'
#     label_path = 'C:/Users/osy04/Desktop/wok_me/project/yolo/labels_bbox/val2017'
#     num_classes = 80
#     high_stride = 20
#     low_stride = 5
#     img_size = 360
#     batch_size = 32
    
#     # Create the generator
#     train_gen = data_generator(num_classes, high_stride, low_stride, img_size,
#                                data_path, label_path, batch_size=1, shuffle=True)
    
#     x_batch, [y_batch_high, y_batch_low] = next(train_gen)
#     print("x_batch shape:", x_batch.shape)
#     print("y_batch_high shape:", y_batch_high.shape)
#     print("y_batch_low shape:", y_batch_low.shape)

#     # Create the tf.data.Dataset
#     train_dataset = create_tf_dataset(num_classes, high_stride, low_stride, img_size,
#                                       data_path, label_path, batch_size=batch_size, shuffle=True)
    

#     coco_cls_names = ['person', 'bicycle', 'car', 'motorcycle', 'airplane', 'bus', 'train', 'truck', 'boat',
#                 'traffic light', 'fire hydrant', 'stop sign', 'parking meter', 'bench', 'bird', 'cat', 'dog',
#                 'horse', 'sheep', 'cow', 'elephant', 'bear', 'zebra', 'giraffe', 'backpack', 'umbrella',
#                 'handbag', 'tie', 'suitcase', 'frisbee', 'skis', 'snowboard', 'sports ball', 'kite', 'baseball bat',
#                 'baseball glove', 'skateboard', 'surfboard', 'tennis racket', 'bottle', 'wine glass', 'cup', 'fork',
#                 'knife', 'spoon', 'bowl', 'banana', 'apple', 'sandwich', 'orange', 'broccoli', 'carrot', 'hot dog',
#                 'pizza', 'donut', 'cake', 'chair', 'couch', 'potted plant', 'bed', 'dining table', 'toilet', 'tv',
#                 'laptop', 'mouse', 'remote', 'keyboard', 'cell phone', 'microwave', 'oven', 'toaster', 'sink',
#                 'refrigerator', 'book', 'clock', 'vase', 'scissors', 'teddy bear', 'hair drier', 'toothbrush']
    
#     for i in range(10):
#         x_data, [y_data_high, y_data_low] = next(iter(train_gen))
#         print(x_data.shape, y_data_high.shape, y_data_low.shape)
#         box = labelDecode.decode(y_data_high[0], y_data_low[0], high_stride, low_stride, conf_threshold=0.3, original_image=True)
#         print(box)
#         draw_result_imshow(np.array(x_data[0]), box, coco_cls_names)
# # #%%




#%%
import numpy as np

# box to model output type
def encode(boxes, num_classes, high_stride, low_stride, area_threshold):

    # 각 해상도별 그리드 셀 크기 계산
    depth = 5 + num_classes  # 4 좌표 + 1 objectness + num_classes
    
    # 각 해상도별 라벨 텐서 초기화
    high_label = np.zeros((high_stride, high_stride, depth), dtype=np.float32)
    low_label  = np.zeros((low_stride, low_stride, depth), dtype=np.float32)
    
    # 박스가 하나도 없는 경우 빈 라벨 텐서 반환
    if boxes is None:
        return high_label, low_label

    # 각 박스에 대해 라벨 텐서에 할당
    for box in boxes:
        class_id = int(box[0])
        x = box[1]
        y = box[2]
        w = box[3]
        h = box[4]
        area = w * h  # normalized area

        # if area is too small, then continue
        if area < 0.0003:
            continue
        
        if area < area_threshold:
            # 작은 객체 → 높은 해상도 head 전용
            cell_x = int(x * high_stride) # 그리드 셀 x 좌표
            cell_y = int(y * high_stride) # 그리드 셀 y 좌표
            offset_x = x * high_stride - cell_x # 그리드 셀 내 x 좌표
            offset_y = y * high_stride - cell_y # 그리드 셀 내 y 좌표

            # 이미 할당된 객체가 있다면, 더 큰 영역의 박스로 덮어쓰기
            if high_label[cell_y, cell_x, 4] == 1.0:
                existing_w = high_label[cell_y, cell_x, 2]
                existing_h = high_label[cell_y, cell_x, 3]
                existing_area = existing_w * existing_h
                if area < existing_area:
                    continue  # 이미 더 큰 박스가 할당되어 있으면 건너뜀

            high_label[cell_y, cell_x, 0] = offset_x
            high_label[cell_y, cell_x, 1] = offset_y
            high_label[cell_y, cell_x, 2] = w
            high_label[cell_y, cell_x, 3] = h
            high_label[cell_y, cell_x, 4] = 1.0  # objectness
            high_label[cell_y, cell_x, 5:] = 0.0   # 기존 클래스 one-hot 초기화
            high_label[cell_y, cell_x, 5 + class_id] = 1.0
            
        else:
            # 큰 객체 → 낮은 해상도 head 전용
            cell_x = int(x * low_stride)
            cell_y = int(y * low_stride)
            offset_x = x * low_stride - cell_x
            offset_y = y * low_stride - cell_y

            if low_label[cell_y, cell_x, 4] == 1.0:
                existing_w = low_label[cell_y, cell_x, 2]
                existing_h = low_label[cell_y, cell_x, 3]
                existing_area = existing_w * existing_h
                if area < existing_area:
                    continue

            low_label[cell_y, cell_x, 0] = offset_x
            low_label[cell_y, cell_x, 1] = offset_y
            low_label[cell_y, cell_x, 2] = w
            low_label[cell_y, cell_x, 3] = h
            low_label[cell_y, cell_x, 4] = 1.0
            low_label[cell_y, cell_x, 5:] = 0.0
            low_label[cell_y, cell_x, 5 + class_id] = 1.0

    return high_label, low_label








# Example usage
if __name__ == "__main__":

    boxes_example = [
        [0.0, 0.076,0.076,0.7,0.7],
        [3.0, 0.53671875, 0.3578125, 0.0859375, 0.109375], 
        [0.0, 0.49921875, 0.93984375, 0.9296875, 0.2390625]
    ]

    high_label, low_label = encoded = encode(boxes_example, num_classes=5, high_stride=23, low_stride=3, area_threshold=0.08)
    print("High resolution label shape:", high_label.shape)
    print("Low resolution label shape:", low_label.shape)




def load_data_from_txt(file_path):
    data = []

    try:
        with open(file_path, 'r') as f:
            for line in f:
                # Strip whitespace and split by spaces
                row_strs = line.strip().split()
                # Convert each element to float
                row_floats = [float(value) for value in row_strs]
                data.append(row_floats)
        return data
    
    # if no file found, return None
    except:
        return None




# Example usage:
if __name__ == '__main__':
    file_path = './imgs/10000_64_4420756.txt'
    data_array = load_data_from_txt(file_path)

    # Print each row
    for row in data_array:
        print(row)